// rsp.h
//  Library interface for C modules to 'talk' to the RSP daemon

// Maximum possible number of elements
#define MAX_LIST_ELMTS		1024

// Longest possible error
#define RSP_ERROR_SIZE		1024

enum
{
	RSP_NOERR = 0,
	RSP_INVALID_ARGS,
	RSP_INVALID_PRIORITY,
	RSP_NO_THRESHOLDS,
	RSP_INVALID_THRESHOLD,
	RSP_RSPD_DOWN,
	RSP_LOW_MEM,
	RSP_BAD_CONVERT,
	RSP_NO_CONFIG_FOUND,
};

enum
{
	RSP_NOTE = 0,
	RSP_WARN,
	RSP_ERROR,
};

enum
{
	GRAPH_NONE = 0,
	GRAPH_TIME,
	GRAPH_SMALLTIME,
	GRAPH_BIGSIZE,
	GRAPH_SMALLSIZE,
	GRAPH_PERCENT,
	GRAPH_BOOL,
	GRAPH_MISC,
};

typedef struct thresholdNode
{
	char* threshold;
	int bCrossed;
	int bForceAlert;
} thresholdNode;

typedef struct _RSP_configElement
{
	char* name;
	char* dataList[MAX_LIST_ELMTS];
	int dataLen;
	int dataCurr;
} RSP_configElement;

typedef struct _RSP_moduleBackend
{
	char* dataToSend;
	int iPollTime;

	struct _RSP_graphReturnData* graphDataToSend[MAX_LIST_ELMTS];
	int graphLen;
	
	struct thresholdNode* thresholds[MAX_LIST_ELMTS];
	int threshLen;
	int threshCurr;

	struct _RSP_configElement* configs[MAX_LIST_ELMTS];
	int configLen;
	int configCurr;
} RSP_moduleBackend;

typedef struct _RSP_graphReturnData
{
	int iType;
	char* szKey;
	char* szData;
} RSP_graphReturnData;

int rsp_error;
char rsp_errorstr[RSP_ERROR_SIZE];

RSP_moduleBackend* RSP_init(char* name);
int RSP_log(RSP_moduleBackend* mod, int priority, char *message, ...);
int RSP_waitNextEvent(RSP_moduleBackend* mod, int secs);
char* RSP_getConfigValue(RSP_moduleBackend* mod, char* name);
RSP_configElement* RSP_getConfig(RSP_moduleBackend* mod, char* name);
char* RSP_nextConfigData(RSP_configElement* config);
char* RSP_nextConfigName(RSP_moduleBackend* mod);
char* RSP_nextThreshold(RSP_moduleBackend* mod);
int RSP_giveData(RSP_moduleBackend* mod, char* data);
int RSP_setThreshold(RSP_moduleBackend* mod, char* thresholdToSet, int forceAlert);
int RSP_giveGraphData(RSP_moduleBackend* mod, RSP_graphReturnData* graphData);
int RSP_postMessage(RSP_moduleBackend* mod);
int RSP_getPolltime(RSP_moduleBackend* mod);
void RSP_cleanup(RSP_moduleBackend* mod);

long RSP_timeConvert(char* data);
double RSP_smallTimeConvert(char* data);
double RSP_bigSizeConvert(char* data);
long RSP_smallSizeConvert(char* data);
float RSP_percentConvert(char* data);
int RSP_boolConvert(char* data);
