#!/usr/bin/perl

use strict;
use lib "modules/rsp-interfaces/perl";
use rspmod;
use Proc::ProcessTable;

my $mod;

sub update() {
	my $t = new Proc::ProcessTable;

	my $argument = $mod->getConfigValue("procCheck");

	if(!defined($argument)) {
		my $count = scalar(@{$t->table});
		my $proccount = "$count processes running";

		$mod->giveData($proccount);

		my $graphData = new rspmod::RSP_graphReturnData(0, "count", "$count");
		$mod->giveGraphData($graphData);

		while(defined(my $threshold = $mod->nextThreshold())) {
			if($threshold =~ /^\s*count\s*([!=<>]=?)\s*(.*)/) {
				if($1 eq "<") {
					$mod->setThreshold($threshold) if $count < $2;
				} elsif($1 eq "<=") {
					$mod->setThreshold($threshold) if $count <= $2;
				} elsif($1 eq ">") {
					$mod->setThreshold($threshold) if $count > $2;
				} elsif($1 eq ">=") {
					$mod->setThreshold($threshold) if $count >= $2;
				} elsif($1 eq "!=") {
					$mod->setThreshold($threshold) if $count != $2;
				} elsif($1 eq "==") {
					$mod->setThreshold($threshold) if $count == $2;
				}
			}
		}
	}
	else {
		my $proccount;
		foreach my $arg (split(/[ |\t]/, $argument)) {
			my $count = 0;
			foreach my $p (@{$t->table}) {
	        $count++ if $p->cmndline =~ /$arg/;
			}

			$proccount .= "$count instances of $arg, ";

			my $graphData = new rspmod::RSP_graphReturnData(0, $arg, $count);
			$mod->giveGraphData($graphData);

			while(defined(my $threshold = $mod->nextThreshold())) {
				if($threshold =~ /$arg\s+(.)\s+(.*)/) {
					if($1 eq "<") {
						$mod->setThreshold($threshold) if $count < $2;
					} elsif($1 eq "<=") {
						$mod->setThreshold($threshold) if $count <= $2;
					} elsif($1 eq ">") {
						$mod->setThreshold($threshold) if $count > $2;
					} elsif($1 eq ">=") {
						$mod->setThreshold($threshold) if $count >= $2;
					} elsif($1 eq "!=") {
						$mod->setThreshold($threshold) if $count != $2;
					} elsif($1 eq "==") {
						$mod->setThreshold($threshold) if $count == $2;
					}
				}
			}
		}

		$mod->giveData(substr($proccount, 0, length($proccount) - 2));
	}

	$mod->postMessage();
}

$mod = new rspmod::RSP_moduleBackend("ProcCount");

while(1) {
	my $returnVal;

	eval {
		$returnVal = $mod->waitNextEvent($mod->getPolltime());
	};

	if($@) {
		if($returnVal == -1) {
			$mod->log(1, "Message from rspd was not recieved correctly.  Quitting");
		}

		exit();
	}

	update();
}

