// rsplib.h

#include <time.h>

// Maximum possible number of elements
#define MAX_LIST_ELMTS		1024

// Longest possible error
#define RSP_ERROR_SIZE		1024

// Error return values
enum
{
	RSP_NOERR = 1,
	RSP_SOCK_CREATE_FAILED,
	RSP_CONNECT_FAILED,
	RSP_LOST_CONNECTION,
	RSP_INVALID_ARGS,
	RSP_NEED_MORE_DATA,
	RSP_HOSTNAME_ERROR,
	RSP_MYSQL_INVALID,
	RSP_LOW_MEM,
	RSP_DUP_PRODKEY,
	RSP_ALREADY_CONNECTED,
	RSP_NO_SSL,
	RSP_SSL_CTX,
	RSP_SSL_KEYGEN,
	RSP_SSL_KEYFILE_OPEN,
	RSP_SSL_KEYFILE_WRITE,
	RSP_SSL_KEYPAIR,
	RSP_SSL_CERT,
	RSP_SSL_CERT_SIGN,
	RSP_SSL_PRIVATE_KEY,
	RSP_SSL_SETUP_CONN,
	RSP_SSL_INVALID_CERT,
	RSP_REDUCED_MODE,
};

enum
{
	PARSE_REMOTE = 0,
	PARSE_MYSQL,
};

enum
{
	GRAPH_NONE = 0,
	GRAPH_TIME,
	GRAPH_SMALLTIME,
	GRAPH_BIGSIZE,
	GRAPH_SMALLSIZE,
	GRAPH_PERCENT,
	GRAPH_BOOL,
};

// structure for a connection
typedef struct connectionData
{
	int taken;
	int secure;

	int sock;

	SSL* ssl;
} connectionData;

typedef struct _RSP_server
{
	struct connectionData* conn;

	SSL_CTX* ctx;

	char key[33];

	char* ip;
	int port;
	int timeout;
	int persistent;

	struct _RSP_module* serverData[MAX_LIST_ELMTS];
	int len;
	int curr;
} RSP_server;

typedef struct _RSP_module
{
	char *name;
	char *data;
	int threshCrossed;
} RSP_module;

// Describes a parse object.  The elements describe how to get the data, as
// well as which data should be returned.  This data structure shouldn't have
// to be directly accessed from anywhere but rsplib.c (therefore creating a
// layer of abstraction).  Not all elements will need to be used.  
typedef struct _RSP_parseData
{
	int iType;							// Connection type: PARSE_REMOTE or PARSE_MYSQL
	char* pszIP;						// ip to connect to
	int iPort;							// port to connect to
	int timeout;						// connection timeout
	int secure;							// Only for remote: connect securely
	SSL_CTX* ctx;
	char* pszLogin;					// Only for database: login, password, and database to use
	char* pszPassword;
	char* pszDatabase;
	int bDataString;					// If we want the data string in addition to graph data
	void* curr;							// The value depends on the type, but this variable is used to keep track of the current position
	struct _RSP_parseLine *currLine;
} RSP_parseData;

// Describes a single "line" returned which matches the given data.
typedef struct _RSP_parseLine
{
	time_t time;
	char* szData;
	char* szGraphData;
	int iType;
} RSP_parseLine;

int rsp_error;
char rsp_errorstr[RSP_ERROR_SIZE];

// Functions to retrieve data from servers
RSP_server* RSP_newServer(char* host, int port, int timeout, int persistent);
int RSP_serverUseSSL(RSP_server* server, char* keyfile);
int RSP_serverConnect(RSP_server* server);
int RSP_serverSendKeepAlive(RSP_server* server);
RSP_module* RSP_nextServerModule(RSP_server* server);
void RSP_serverFree(RSP_server* server);

// Functions to retrieve history data locally/remotely/from a database
RSP_parseData* RSP_newParseRemote(char* szHost, int iPort, int iTimeout);
RSP_parseData* RSP_newParseMySQL(char* szHost, int iPort, char* szLogin, char* szPassword, char* szDatabase, int iTimeout);
int RSP_remoteUseSSL(RSP_parseData* pd, char* keyfile);
int RSP_getHostsAndModules(RSP_parseData* pd, char*** pszHosts, char*** pszModules);
int RSP_getKeys(RSP_parseData* pd, char* szHost, char* szModule, char*** pszKeys);
int RSP_parseBegin(RSP_parseData* pd, char* szHost, char* szModule, char* szKey, time_t beginTime, time_t endTime, int bDataString);
RSP_parseLine* RSP_parseNext(RSP_parseData* pd);
void RSP_parseDelete(RSP_parseData* pd);
