#!/usr/bin/perl -w
# Description
#   This script will install necessary files into your Perl library path so you
#   can reference the RSP Perl binding interfaces.
#
# Usage
#   - To install in standard locations, type ./install_perl_files.pl
#   - To install in a custom location, use 
#       ./install_perl_files.pl --prefix={DIR} 
# 
# Copyright 2004 Draconis Software, LLC.  All rights reserved.  For more 
# information about RSP and Draconis Software, please see our website at 
# http://www.dracoware.com

use strict;
use File::Copy;

my @FILES = qw(rsplib.pm rsplib.so);

sub get_location()
{
	foreach my $dir (@INC) {
		return $dir if check_dir($dir);
	}
	print "No acceptable directories in \@INC. Do you have write access?\n";
	exit 0;
}

sub install_files($)
{
	my $loc = shift @_;
	print "Installing files to $loc\n";
	
	foreach my $file (@FILES) {
		copy($file, "$loc/$file") or die "Failed to copy: $!";
		print "cp $file $loc/$file\n";
	}
}

sub check_dir($)
{
	my $dir = shift @_;
	return 0 if !$dir;
	return 1 if -d $dir;
	return 0;
}

sub main()
{
	my $loc;
	
	# check user arguments
	if(($#ARGV + 1) > 0)
	{
		foreach my $arg (@ARGV)
		{
			if($arg eq "--help" || $arg eq "-h") {
				print "install_perl_files.pl Help\n";
				print "This script will install necessary files into your Perl library path so you\n";
				print "can reference the RSP Perl binding interfaces.\n";
				print "\n";
				print "  - To install in standard locations, type ./install_perl_files.pl\n";
				print "  - To install in a custom location, use ./install_perl_files.pl --prefix={DIR}\n";
				exit 0;
			}
			elsif(substr($arg, 0, 9) eq "--prefix=") {
				$loc = substr($arg, 9);
				if($loc eq "") {
					print "Argument '$arg' was not understood.\n";
					print "Type ./install_perl_files.pl --help for usage information\n";
					exit 0;
				}
			}
			else {
				print "Argument '$arg' was not understood.\n";
				print "Type ./install_perl_files.pl --help for usage information\n";
				exit 0;
			}
		}
	}
	
	# install files
	if(check_dir($loc) == 1) {
		# install into user-specified directory
		install_files($loc);
	}
	else {
		# install into a standard directory
		install_files(get_location());
	}
}

main();

