/* rspinventory.js
 *
 * RSP Inventory Javascript sources
 */
var currCriteriaState = new Array();
var newCriteriaPanelId = -1;
var currSortBy = "Name";
var currSortByAscending = 0;

currCriteriaState[0] = new Array(criteriaKeys[0], null);

// preload the criteria images for speed
if (document.images)
{
	addDefault = new Image(18, 18);
	addDefault.src = "images/add_uns.gif";
	addHover = new Image(18, 18);
	addHover.src = "images/add.gif";
	addDown = new Image(18, 18);
	addDown.src = "images/add.gif";
	
	deleteDefault = new Image(18, 18);
	deleteDefault.src = "images/delete_uns.gif";
	deleteHover = new Image(18, 18);
	deleteHover.src = "images/delete.gif";
	deleteDown = new Image(18, 18);
	deleteDown.src = "images/delete.gif";
}

function pollBackendStatus()
{
	getLatestInventoryHosts();
	setTimeout("pollBackendStatus()", 5000);
}

/* swapCriteriaImg
 *
 * Called for mouseOver and mouseOut on the add/delete buttons.
 */
function swapCriteriaImg(imgName, action, num)
{
	if (document.images)
	{
		if (action == "hover")
		{
			if((imgName.indexOf('add') > -1) && (currCriteriaState.length < criteriaKeys.length) ||
				((imgName.indexOf('delete') > -1) && (currCriteriaState.length > 1)))
			{
				imgOn = eval(imgName + "Hover.src");
				document[imgName + num].src = imgOn;
			}
		}
		else if (action == "off") {
			imgOff = eval(imgName + "Default.src");
			document[imgName + num].src = imgOff;
		}
		else if (action == "click") {
			img = eval(imgName + "Down.src");
			document[imgName + num].src = img;
		}
	}
}

/* checkOnClickValueInputBox
 *
 * When the user clicks to edit the criteriaValue input box, removes the contents 
 * if nothing has been previously set (i.e. "(anything)").
 */
function checkOnClickValueInputBox(ibox)
{
	if(ibox.value == "(anything)") {
		ibox.style.color = "#000000";
		ibox.value = "";
	}
}

/* checkOnClickValueInputBox
 *
 * If the user deletes the contents of the criteriaValue input box, then replaces the 
 * contents with the string "(anything)", and updates the backend.
 */
function checkOutClickValueInputBox(ibox, num)
{
	if(ibox.value == "")
	{
		currCriteriaState[num-1][1] = null;
		
		ibox.style.color = "#808080";
		ibox.value = "(anything)";
	}
	else
		currCriteriaState[num-1][1] = ibox.value;
}

/* criteriaValueOnChange
 *
 * Called when the user makes a change to a criteriaValue box.  Updates the current
 * criteria state and the inventory table.
 */
function criteriaValueOnChange(ibox, num)
{
	if(ibox.value == "")
		currCriteriaState[num-1][1] = null;
	else
		currCriteriaState[num-1][1] = ibox.value;
	
	getLatestInventoryTable();
}

/* switchCriteriaKey
 *
 * Called when a user clicks on a key drop-down and changes it (i.e. from "os" to "ram").
 * We update the currCriteriaState with the new value and have the Controller rerender 
 * the criteria panels.
 */
function switchCriteriaKey(keyNum, key)
{
	var criteriaValue = document.getElementById('criteriaValue' + keyNum).value;
	if(criteriaValue == '(anything)')
		criteriaValue = null;
	
	currCriteriaState[keyNum-1] = new Array(key.value, criteriaValue);
	
	getLatestCriteria();
	getLatestInventoryTable();
}

/* setInventoryTableSortBy
 *
 * Called when a user clicks an inventory table column.  Resorts the table using the 
 * column's name (key) as basis - switches between ascending & descending.
 *
 * colName - the column name the user clicked
 */
function setInventoryTableSortBy(colName)
{
	if((currSortBy == colName) && (currSortByAscending == 1))
		currSortByAscending = 0;
	else
		currSortByAscending = 1;
	
	currSortBy = colName;
	
	// update the inventory table
	getLatestInventoryTable();
}

/* getCurrCriteriaState
 *
 * Serializes the currCriteriaState array into a string
 */
function getCurrCriteriaState()
{
	var currState = "";				
	currCriteriaState.each(function(value, index) {
		currState += value[0] + ':' + value[1] + "\n";
	});
	
	return(currState);
}

/* criteriaContained
 *
 * Given a string, searches the currCriteriaState array and returns true if 
 * the given string represents a key in the currCriteriaState, and false 
 * otherwise.
 */
function criteriaContained(key)
{
	var returnVal = false;
	
	currCriteriaState.each(function(value, index) {
		if(value[0] == key)
			returnVal = true;
	});
	
	return returnVal;
}

/* addCriteria
 *
 * Called when the user wants to add a new criteria key
 */
function addCriteria(num)
{
	var nextCriteriaKey = null;
	
	// find the next unused criteria key
	for(i = num; i < criteriaKeys.length; i++)
	{
		if(!criteriaContained(criteriaKeys[i])) {
			nextCriteriaKey = criteriaKeys[i];
			break;
		}
	}
	
	if(nextCriteriaKey == null)
	{
		// attempt to start at beginning
		for(i = 0; i < num; i++)
		{
			if(!criteriaContained(criteriaKeys[i])) {
				nextCriteriaKey = criteriaKeys[i];
				break;
			}			
		}
	}
	
	if(nextCriteriaKey != null)
	{
		// we found an unused key
		currCriteriaState.splice(num, 0, new Array(nextCriteriaKey, null));
		newCriteriaPanelId = num;
		
		getLatestCriteria();
		getLatestInventoryTable();
	}
}

/* deleteCriteria
 *
 * Called when the user wants to delete a criteria panel
 */
function deleteCriteria(num)
{
	// ensure there is at least 2 criteria to start with
	if(currCriteriaState.length > 1)
	{
		currCriteriaState.splice(num-1, 1);
		Effect.SlideUp('criteriaPanel' + num, { duration: 0.6 });
		setTimeout('deleteCriteriaUpdatePage()', 600);
	}
}

/* deleteCriteriaUpdatePage
 *
 * Called from deleteCriteria() to run the update.  This is necessary as we use a setTimeout
 * so we might animate the "sliding up" of the content panel being deleted before we call the 
 * AJAX code to replace it.  Necessary as scriptaculous doesn't block.
 */
function deleteCriteriaUpdatePage()
{
	getLatestCriteria();
	getLatestInventoryTable();
}

/* pageSetup
 *
 * Called when the inventory page is first displayed.
 */
function pageSetup()
{
	getLatestCriteria();
	getLatestInventoryTable();
	
	pollBackendStatus();
}

/* getLatestCriteria
 *
 * AJAX updater to retrieve updated criteria HTML from backend.
 */
function getLatestCriteria()
{
	// redraw the criteria HTML
	var url = inventoryContentHTML;	
	var pars = 'action=getCriteria&state=' + getCurrCriteriaState();
	if(newCriteriaPanelId > -1)
		pars += "&newPanelId=" + newCriteriaPanelId;
	
	var myAjax = new Ajax.Request(url, { method: 'post', parameters: pars, onComplete: updateCriteria });
}

/* updateCriteria
 *
 * Callback via Ajax to change the criteriaContainer layer to hold the updated 
 * criteria HTML.
 */
function updateCriteria(originalRequest)
{
	document.getElementById('criteriaContainer').innerHTML = originalRequest.responseText;
	
	// add each of the select'able elements to the criteriaPanel keysList boxes
	var i = 1;
	currCriteriaState.each(function(value, index)
	{
		var thisKey = value[0];
		
		criteriaKeys.each(function(value, index)
		{
			// if the key is this criteria panel's, add it (and set it as selected)
			// otherwise, if the key is in the list of available keys, and isn't selected
			// anywhere else, then add it deselected
			if(thisKey == value) {
				document.getElementById("keyList" + i).options[document.getElementById("keyList" + i).length] = new Option(value, value);
				document.getElementById("keyList" + i).options[document.getElementById("keyList" + i).length - 1].selected = true;
			}
			else if(!criteriaContained(value))
				document.getElementById("keyList" + i).options[document.getElementById("keyList" + i).length] = new Option(value, value);
		});
		
		i++;
	});
	
	// do a slide down effect for the new panel
	if(newCriteriaPanelId > -1) {
		Effect.SlideDown('criteriaPanel' + (newCriteriaPanelId+1), { duration: 0.6 });
		newCriteriaPanelId = -1;
	}
}

/* getLatestInventoryTable()
 *
 * AJAX updater to retrieve latest inventory table HTML from backend.
 */
function getLatestInventoryTable()
{
	// redraw the criteria HTML
	var url = inventoryContentHTML;
	var pars = 'action=getInventoryTable&state=' + getCurrCriteriaState() + "&sortByCol=" + currSortBy + "&asc=" + currSortByAscending;
	var myAjax = new Ajax.Request(url, { method: 'post', parameters: pars, onComplete: updateInventoryTableContent });
}

/* updateInventoryTableContent
 *
 * Callback via Ajax to change the inventoryTable layer to hold the updated 
 * inventory table (as set by the criteria).
 */
function updateInventoryTableContent(originalRequest)
{
	document.getElementById('inventoryTable').innerHTML = originalRequest.responseText;
}

/* getLatestInventoryHosts
 *
 * Requests the number of hosts in our inventory db that we currently have.
 */
function getLatestInventoryHosts()
{
	// redraw the criteria HTML
	var url = inventoryContentHTML;
	var pars = 'action=getInventoryHosts';
	var myAjax = new Ajax.Request(url, { method: 'post', parameters: pars, onComplete: updateInventoryHostsContent });
}

/* updateInventoryHostsContent
 *
 * Updates the inventoryBackendStatus div with how many hosts 
 * we currently have in our inventory db
 */
function updateInventoryHostsContent(originalRequest)
{
	document.getElementById('inventoryBackendStatus').innerHTML = originalRequest.responseText;
}
